using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Marketing;
using Vevo.Shared.Utilities;
using Vevo.Domain.Stores;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class NewsLetterRepository : INewsLetterRepository
    {
        #region Private

        private const string RepositoryName = "NewsLetterRepository";

        private const string GetOneSignature = "GetOne";
        private const string SearchNewsletterSignature = "SearchNewsletter";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string GetAllSignature = "GetAll";
        private const string GetAllByStoreSignature = "GetAllByStore";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "Newsletter", "Customer","Store" } );

        private NewsLetterRepositoryNoCache _repository = new NewsLetterRepositoryNoCache();

        #endregion

        #region Constructors

        public NewsLetterRepository()
        {
        }

        public NewsLetterRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public NewsLetter Create( NewsLetter newsLetter )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            NewsLetter newNewsLetter = _repository.Create( newsLetter );
            _dataCache.ClearRepositoryCache();
            return newNewsLetter;
        }

        public NewsLetter Update( NewsLetter newsLetter, string newEmail ,string newStoreID )
        {
            NewsLetter newNewsLetter = _repository.Update( newsLetter, newEmail, newStoreID );
            _dataCache.ClearRepositoryCache();
            return newNewsLetter;
        }

        public bool DeleteEmail( string emailHash, Store store )
        {
            bool result = _repository.DeleteEmail( emailHash, store );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public bool DeleteEmailNoHash( string emailName, Store store )
        {
            bool result = _repository.DeleteEmailNoHash( emailName, store );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public bool DeleteEmailByStore( Store store )
        {
            bool result = _repository.DeleteEmailByStore( store );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public NewsLetter GetOne( string email, Store store )
        {
            NewsLetter result = (NewsLetter) _dataCache.GetValue( GetOneSignature, _cacheDependency, email, store.StoreID);

            if (result == null)
            {
                result = _repository.GetOne( email, store );
                _dataCache.SetValue( GetOneSignature, result, email, store.StoreID );
            }

            return result;
        }

        public IList<NewsLetter> GetAll( string sortBy )
        {
            IList<NewsLetter> list = (IList<NewsLetter>) _dataCache.GetValue(
                GetAllSignature,
                _cacheDependency,
                sortBy );

            if (list == null)
            {
                list = _repository.GetAll( sortBy );
                _dataCache.SetValue( GetAllSignature, list, sortBy );
            }

            return list;
        }

        public IList<NewsLetter> GetAllByStore( Store store )
        {
            IList<NewsLetter> list = (IList<NewsLetter>) _dataCache.GetValue(
                GetAllByStoreSignature,
                _cacheDependency,
                store.StoreID );

            if (list == null)
            {
                list = _repository.GetAllByStore( store );
                _dataCache.SetValue( GetAllSignature, list, store.StoreID );
            }

            return list;
        }

        public IList<NewsLetter> SearchNewsletter(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchNewsletterSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchNewsletter(
                    sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchNewsletterSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex);
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<NewsLetter>) result.First;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue( GetTableSchemaSignature, list );
            }

            return list;
        }

        #endregion
    }
}
